"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformEIP1193Provider = transformEIP1193Provider;
const viem_1 = require("viem");
const accounts_1 = require("viem/accounts");
const abstractClient_js_1 = require("./abstractClient.js");
const utils_js_1 = require("./utils.js");
function transformEIP1193Provider(options) {
    const { provider, chain, transport: overrideTransport } = options;
    const providerHandleRequest = provider.request;
    const transport = overrideTransport ?? (0, viem_1.custom)(provider);
    const handler = async (e) => {
        const { method, params } = e;
        switch (method) {
            case 'eth_accounts': {
                const accounts = await provider.request({ method: 'eth_accounts' });
                const publicClient = (0, viem_1.createPublicClient)({
                    chain,
                    transport,
                });
                if (accounts?.[0] === undefined) {
                    return [];
                }
                const smartAccount = await (0, utils_js_1.getSmartAccountAddressFromInitialSigner)(accounts[0], publicClient);
                return [smartAccount, accounts[0]];
            }
            case 'eth_signTransaction':
            case 'eth_sendTransaction': {
                const accounts = await provider.request({ method: 'eth_accounts' });
                const account = accounts[0];
                if (!account) {
                    throw new Error('Account not found');
                }
                const transaction = params[0];
                if (transaction.from === account) {
                    return await providerHandleRequest(e);
                }
                const wallet = (0, viem_1.createWalletClient)({
                    account,
                    transport,
                });
                const signer = (0, accounts_1.toAccount)({
                    address: account,
                    signMessage: wallet.signMessage,
                    signTransaction: wallet.signTransaction,
                    signTypedData: wallet.signTypedData,
                });
                const abstractClient = await (0, abstractClient_js_1.createAbstractClient)({
                    chain,
                    signer,
                    transport,
                });
                if (transaction.eip712Meta && transaction.eip712Meta.paymasterParams) {
                    transaction.paymaster =
                        transaction.eip712Meta.paymasterParams.paymaster;
                    transaction.paymasterInput = (0, viem_1.toHex)(transaction.eip712Meta.paymasterParams.paymasterInput);
                }
                if (method === 'eth_signTransaction') {
                    return (await abstractClient.signTransaction(transaction));
                }
                else if (method === 'eth_sendTransaction') {
                    return await abstractClient.sendTransaction(transaction);
                }
                throw new Error('Should not have reached this point');
            }
            default: {
                return await providerHandleRequest(e);
            }
        }
    };
    return {
        ...provider,
        on: provider.on,
        removeListener: provider.removeListener,
        request: handler,
    };
}
//# sourceMappingURL=transformEIP1193Provider.js.map